//////////////////////////////////////////////////////////////////////
// IP traffic analyzer
// Written by Jonathan A. Foster <ChipMaster@YeOlPiShack.net>
// Started April 23rd, 2021
//
// The idea is to analyze iptables LOG entries in combination with
// DNSmasq's query log entries and combine them to list the hosts
// that were accessed. The main reasons for not just inspecting HTTP
// packets through a netfilter socket is due to HTTPS hiding the
// "host" field. So I'm deducing based on DNS query timing.
//
// NOTE: its assumed that the log being processed is in chronological
// order. This is the usual way things get logged. ;-)
//
// 2021-05-14 <ChipMaster@YeOlPiShack.net>
//   Dumbed down for C++ < 11.
//   Split into modules
//////////////////////////////////////////////////////////////////////

// TODO: map names according to time and host. time is probably automatic

#include <string.h>
#include <string>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <vector>
#include <map>
#include "strutil.h"
#include "data.h"
#include "config.h"
using namespace std;



//////////////////////////////////////////////////////////////////////
// Busy indicator aka. "Live Bug"
//////////////////////////////////////////////////////////////////////

struct LiveBug {
  string seq;
  char   pre;
  int    p;
  LiveBug(): seq("-\\|/"), pre('\r'), p(0) {}
  inline char next() { if(p>=seq.size()) p=0; return seq[p++]; }
};
ostream &operator<<(ostream &o, LiveBug &bug) {
  return o << bug.pre << bug.next();
}



//////////////////////////////////////////////////////////////////////
// Roll through file
//////////////////////////////////////////////////////////////////////
//#define DEBUG

// (char *) to silence warnings...
const char *dns_ignore[] = { (char *)"v=spf1", (char *)"https:", 0 };
const char *dns_del[]    = { (char *)"NODATA-", (char *)"NXDOMAIN-", 0 };
#define PATH "/srv/backups/iptraffic"
ifstream log(PATH "/test.log");
ofstream out(PATH "/processed.log");
Splits ln;
int lnno = 0, ict = 0;
LiveBug bug;
NameVal rdns;
NameVal::iterator nvp;
string name, address, s;
Conn conn;
bool match;
Config config;



void dlog(const string msg) {
#ifdef DEBUG
  cerr << "\r" << lnno << ": " << msg << endl;
#endif
}



int main(int argc, char **argv) {

  /// Load config ///

  config.load(PATH "/iptraffic.conf");

  /// parse log file ///

  while((log >> ln)) {
    lnno++;
    cout << bug << " " << lnno << flush;

    ///  DNS query result ///

    // TODO: need to get more specific on tying us + them + time to DNS
    if(ln.count>8 && strncmp(ln.fields[4], "dnsmasq[", 8)==0) {
      if(ln[5]=="reply") {
        name = ln[6];
        address = ln[8];
        // Hmm... is this reply an address?
        if(pre_match(dns_ignore, address)) continue; // nope
        if(pre_match(dns_del,    address)) continue; // does not exist reply
        if((nvp=rdns.find(address))!=rdns.end()) {
          if(nvp->second==name) continue;
          dlog("WARN: DNS address overlap "+address+": "+nvp->second+" : "+name);
        }
        rdns[address] = name;
        dlog("Added "+address+" = "+name);
#ifdef DEBUG
        cout << '\r' << lnno << ": " << name << endl;
#endif
        continue;
      }
    }

    /// process connections ///

    if(ln.count>5
    && ln[4]=="kernel:"
    && ln[5]=="ACCEPT"
    ) {
      conn = ln;
      if(!pre_match(config.us, conn.us)) conn.swap();
      if((nvp=rdns.find(conn.them))!=rdns.end())
        conn.name = nvp->second;
      if(config.ignores.find(conn)<0)
        out << ln[0] << " " << ln[1] << " " << ln[2] << " " << conn << "\n";
      else
        ict++;
    }
  }
  cout << "\nIgnored: " << ict << endl;
  cout << "Total rDNS: " << rdns.size() << "\n";
  return 0;
}
